@@ -2,6 +2,7 @@
 import logging
 import os
 import warnings
+from functools import lru_cache
 
 import dask.array as da
 import iris
@@ -14,21 +15,6 @@ from esmvalcore.iris_helpers import var_name_constraint
 logger = logging.getLogger(__name__)
 
 
-def _get_altitude_to_pressure_func():
-    """Get function converting altitude [m] to air pressure [Pa]."""
-    base_dir = os.path.dirname(os.path.abspath(__file__))
-    source_file = os.path.join(base_dir, 'us_standard_atmosphere.csv')
-    data_frame = pd.read_csv(source_file, comment='#')
-    func = interp1d(data_frame['Altitude [m]'],
-                    data_frame['Pressure [Pa]'],
-                    kind='cubic',
-                    fill_value='extrapolate')
-    return func
-
-
-altitude_to_pressure = _get_altitude_to_pressure_func()  # noqa
-
-
 class AtmosphereSigmaFactory(iris.aux_factory.AuxCoordFactory):
     """Defines an atmosphere sigma coordinate factory."""
 
@@ -42,20 +28,17 @@ class AtmosphereSigmaFactory(iris.aux_factory.AuxCoordFactory):
                         (surface_air_pressure(n, j, i) - pressure_at_top)
 
         """
+        self._metadata_manager = iris.common.metadata_manager_factory(
+            iris.common.CoordMetadata)
         super().__init__()
         self._check_dependencies(pressure_at_top, sigma, surface_air_pressure)
+        self.units = pressure_at_top.units
         self.pressure_at_top = pressure_at_top
         self.sigma = sigma
         self.surface_air_pressure = surface_air_pressure
         self.standard_name = 'air_pressure'
         self.attributes = {}
 
-    @property
-    def units(self):
-        """Units."""
-        units = self.pressure_at_top.units
-        return units
-
     @staticmethod
     def _check_dependencies(pressure_at_top, sigma, surface_air_pressure):
         """Check for sufficient coordinates."""
@@ -77,8 +60,8 @@ class AtmosphereSigmaFactory(iris.aux_factory.AuxCoordFactory):
         # Check bounds
         if sigma.nbounds not in (0, 2):
             raise ValueError(
-                "Invalid 'sigma' coordinate: must have either 0 or 2 bounds, "
-                "got {sigma.nbounds:d}")
+                f"Invalid 'sigma' coordinate: must have either 0 or 2 bounds, "
+                f"got {sigma.nbounds:d}")
         for coord in (pressure_at_top, surface_air_pressure):
             if coord.nbounds:
                 msg = (f"Coordinate '{coord.name()}' has bounds. These will "
@@ -86,15 +69,18 @@ class AtmosphereSigmaFactory(iris.aux_factory.AuxCoordFactory):
                 warnings.warn(msg, UserWarning, stacklevel=2)
 
         # Check units
+        if sigma.units.is_unknown():
+            sigma.units = Unit('1')
         if not sigma.units.is_dimensionless():
             raise ValueError(
-                "Invalid units: 'sigma' must be dimensionless, got "
-                "'{sigma.units}'")
+                f"Invalid units: 'sigma' must be dimensionless, got "
+                f"'{sigma.units}'")
         if pressure_at_top.units != surface_air_pressure.units:
             raise ValueError(
-                "Incompatible units: 'pressure_at_top' and "
-                "'surface_air_pressure' must have the same units, got "
-                "'{pressure_at_top.units}' and '{surface_air_pressure.units}'")
+                f"Incompatible units: 'pressure_at_top' and "
+                f"'surface_air_pressure' must have the same units, got "
+                f"'{pressure_at_top.units}' and "
+                f"'{surface_air_pressure.units}'")
         if not pressure_at_top.units.is_convertible('Pa'):
             raise ValueError(
                 "Invalid units: 'pressure_at_top' and 'surface_air_pressure' "
@@ -257,8 +243,12 @@ def add_plev_from_altitude(cube):
         height_coord = cube.coord('altitude')
         if height_coord.units != 'm':
             height_coord.convert_units('m')
+        altitude_to_pressure = get_altitude_to_pressure_func()
         pressure_points = altitude_to_pressure(height_coord.core_points())
-        pressure_bounds = altitude_to_pressure(height_coord.core_bounds())
+        if height_coord.core_bounds() is None:
+            pressure_bounds = None
+        else:
+            pressure_bounds = altitude_to_pressure(height_coord.core_bounds())
         pressure_coord = iris.coords.AuxCoord(pressure_points,
                                               bounds=pressure_bounds,
                                               var_name='plev',
@@ -272,6 +262,43 @@ def add_plev_from_altitude(cube):
         "available")
 
 
+def add_altitude_from_plev(cube):
+    """Add altitude coordinate from pressure level coordinate.
+
+    Parameters
+    ----------
+    cube : iris.cube.Cube
+        Input cube.
+
+    Raises
+    ------
+    ValueError
+        ``cube`` does not contain coordinate ``air_pressure``.
+
+    """
+    if cube.coords('air_pressure'):
+        plev_coord = cube.coord('air_pressure')
+        if plev_coord.units != 'Pa':
+            plev_coord.convert_units('Pa')
+        pressure_to_altitude = get_pressure_to_altitude_func()
+        altitude_points = pressure_to_altitude(plev_coord.core_points())
+        if plev_coord.core_bounds() is None:
+            altitude_bounds = None
+        else:
+            altitude_bounds = pressure_to_altitude(plev_coord.core_bounds())
+        altitude_coord = iris.coords.AuxCoord(altitude_points,
+                                              bounds=altitude_bounds,
+                                              var_name='alt',
+                                              standard_name='altitude',
+                                              long_name='altitude',
+                                              units='m')
+        cube.add_aux_coord(altitude_coord, cube.coord_dims(plev_coord))
+        return
+    raise ValueError(
+        "Cannot add 'altitude' coordinate, 'air_pressure' coordinate not "
+        "available")
+
+
 def add_scalar_depth_coord(cube, depth=0.0):
     """Add scalar coordinate 'depth' with value of `depth`m."""
     logger.debug("Adding depth coordinate (%sm)", depth)
@@ -345,6 +372,26 @@ def cube_to_aux_coord(cube):
     )
 
 
+@lru_cache()
+def get_altitude_to_pressure_func():
+    """Get function converting altitude [m] to air pressure [Pa].
+
+    Returns
+    -------
+    callable
+        Function that converts altitude to air pressure.
+
+    """
+    base_dir = os.path.dirname(os.path.abspath(__file__))
+    source_file = os.path.join(base_dir, 'us_standard_atmosphere.csv')
+    data_frame = pd.read_csv(source_file, comment='#')
+    func = interp1d(data_frame['Altitude [m]'],
+                    data_frame['Pressure [Pa]'],
+                    kind='cubic',
+                    fill_value='extrapolate')
+    return func
+
+
 def get_bounds_cube(cubes, coord_var_name):
     """Find bound cube for a given variable in a :class:`iris.cube.CubeList`.
 
@@ -382,6 +429,26 @@ def get_bounds_cube(cubes, coord_var_name):
         f"cubes\n{cubes}")
 
 
+@lru_cache()
+def get_pressure_to_altitude_func():
+    """Get function converting air pressure [Pa] to altitude [m].
+
+    Returns
+    -------
+    callable
+        Function that converts air pressure to altitude.
+
+    """
+    base_dir = os.path.dirname(os.path.abspath(__file__))
+    source_file = os.path.join(base_dir, 'us_standard_atmosphere.csv')
+    data_frame = pd.read_csv(source_file, comment='#')
+    func = interp1d(data_frame['Pressure [Pa]'],
+                    data_frame['Altitude [m]'],
+                    kind='cubic',
+                    fill_value='extrapolate')
+    return func
+
+
 def fix_bounds(cube, cubes, coord_var_names):
     """Fix bounds for cube that could not be read correctly by :mod:`iris`.
 
