@@ -325,7 +325,10 @@ class CMIP6Info(InfoBase):
             if dimension in generic_levels:
                 coord = CoordinateInfo(dimension)
                 coord.generic_level = True
-                coord.axis = 'Z'
+                for name in self.coords:
+                    generic_level = self.coords[name].generic_lev_name
+                    if dimension in [generic_level]:
+                        coord.generic_lev_coords[name] = self.coords[name]
             else:
                 try:
                     coord = self.coords[dimension]
@@ -577,6 +580,7 @@ class CoordinateInfo(JsonInfo):
         super(CoordinateInfo, self).__init__()
         self.name = name
         self.generic_level = False
+        self.generic_lev_coords = {}
 
         self.axis = ""
         """Axis"""
@@ -606,6 +610,8 @@ class CoordinateInfo(JsonInfo):
         """Maximum allowed value"""
         self.must_have_bounds = ""
         """Whether bounds are required on this dimension"""
+        self.generic_lev_name = ""
+        """Generic level name"""
 
     def read_json(self, json_data):
         """
@@ -634,6 +640,7 @@ class CoordinateInfo(JsonInfo):
         self.valid_max = self._read_json_variable('valid_max')
         self.requested = self._read_json_list_variable('requested')
         self.must_have_bounds = self._read_json_variable('must_have_bounds')
+        self.generic_lev_name = self._read_json_variable('generic_level_name')
 
 
 class CMIP5Info(InfoBase):
