@@ -16,7 +16,7 @@ class DerivedVariable(DerivedVariableBase):
     @staticmethod
     def calculate(cubes):
         """Compute toa net downward longwave radiation assuming clear sky."""
-        rlutcs_cube = cubes.extract_strict(
+        rlutcs_cube = cubes.extract_cube(
             Constraint(name='toa_outgoing_longwave_flux_assuming_clear_sky'))
         rlutcs_cube.data = -rlutcs_cube.core_data()
         rlutcs_cube.attributes['positive'] = 'down'
