@@ -3,7 +3,6 @@ import copy
 import logging
 import os
 import shutil
-from collections import OrderedDict
 from itertools import groupby
 from warnings import catch_warnings, filterwarnings
 
@@ -300,19 +299,6 @@ def cleanup(files, remove=None):
     return files
 
 
-def _ordered_safe_dump(data, stream):
-    """Write data containing OrderedDicts to yaml file."""
-    class _OrderedDumper(yaml.SafeDumper):
-        pass
-
-    def _dict_representer(dumper, data):
-        return dumper.represent_mapping(
-            yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG, data.items())
-
-    _OrderedDumper.add_representer(OrderedDict, _dict_representer)
-    return yaml.dump(data, stream, _OrderedDumper)
-
-
 def write_metadata(products, write_ncl=False):
     """Write product metadata to file."""
     output_files = []
@@ -325,7 +311,7 @@ def write_metadata(products, write_ncl=False):
                 p.attributes.get('dataset', ''),
             ),
         )
-        metadata = OrderedDict()
+        metadata = {}
         for product in sorted_products:
             if isinstance(product.attributes.get('exp'), (list, tuple)):
                 product.attributes = dict(product.attributes)
@@ -337,7 +323,7 @@ def write_metadata(products, write_ncl=False):
         output_filename = os.path.join(output_dir, 'metadata.yml')
         output_files.append(output_filename)
         with open(output_filename, 'w') as file:
-            _ordered_safe_dump(metadata, file)
+            yaml.safe_dump(metadata, file)
         if write_ncl:
             output_files.append(_write_ncl_metadata(output_dir, metadata))
 
