@@ -41,8 +41,7 @@ for _coord in (
 
 def extract_time(cube, start_year, start_month, start_day, end_year, end_month,
                  end_day):
-    """
-    Extract a time range from a cube.
+    """Extract a time range from a cube.
 
     Given a time range passed in as a series of years, months and days, it
     returns a time-extracted cube with data only within the specified
@@ -74,7 +73,6 @@ def extract_time(cube, start_year, start_month, start_day, end_year, end_month,
     ------
     ValueError
         if time ranges are outside the cube time limits
-
     """
     time_coord = cube.coord('time')
     time_units = time_coord.units
@@ -83,21 +81,21 @@ def extract_time(cube, start_year, start_month, start_day, end_year, end_month,
             start_day = 30
         if end_day > 30:
             end_day = 30
-    t_1 = PartialDateTime(
-        year=int(start_year), month=int(start_month), day=int(start_day))
-    t_2 = PartialDateTime(
-        year=int(end_year), month=int(end_month), day=int(end_day))
+    t_1 = PartialDateTime(year=int(start_year),
+                          month=int(start_month),
+                          day=int(start_day))
+    t_2 = PartialDateTime(year=int(end_year),
+                          month=int(end_month),
+                          day=int(end_day))
 
-    constraint = iris.Constraint(
-        time=lambda t: t_1 <= t.point < t_2)
+    constraint = iris.Constraint(time=lambda t: t_1 <= t.point < t_2)
 
     cube_slice = cube.extract(constraint)
     if cube_slice is None:
         raise ValueError(
             f"Time slice {start_year:0>4d}-{start_month:0>2d}-{start_day:0>2d}"
             f" to {end_year:0>4d}-{end_month:0>2d}-{end_day:0>2d} is outside "
-            f"cube time bounds {time_coord.cell(0)} to {time_coord.cell(-1)}."
-        )
+            f"cube time bounds {time_coord.cell(0)} to {time_coord.cell(-1)}.")
 
     # Issue when time dimension was removed when only one point as selected.
     if cube_slice.ndim != cube.ndim:
@@ -108,9 +106,33 @@ def extract_time(cube, start_year, start_month, start_day, end_year, end_month,
     return cube_slice
 
 
-def extract_season(cube, season):
+def clip_start_end_year(cube, start_year, end_year):
+    """Extract time range given by the dataset keys.
+
+    Parameters
+    ----------
+    cube : iris.cube.Cube
+        Input cube.
+    start_year : int
+        Start year.
+    end_year : int
+        End year.
+
+    Returns
+    -------
+    iris.cube.Cube
+        Sliced cube.
+
+    Raises
+    ------
+    ValueError
+        Time ranges are outside the cube's time limits.
     """
-    Slice cube to get only the data belonging to a specific season.
+    return extract_time(cube, start_year, 1, 1, end_year + 1, 1, 1)
+
+
+def extract_season(cube, season):
+    """Slice cube to get only the data belonging to a specific season.
 
     Parameters
     ----------
@@ -118,24 +140,39 @@ def extract_season(cube, season):
         Original data
     season: str
         Season to extract. Available: DJF, MAM, JJA, SON
+        and all sequentially correct combinations: e.g. JJAS
 
     Returns
     -------
     iris.cube.Cube
         data cube for specified season.
     """
+    season = season.upper()
+
+    allmonths = 'JFMAMJJASOND' * 2
+    if season not in allmonths:
+        raise ValueError(f"Unable to extract Season {season} "
+                         f"combination of months not possible.")
+    sstart = allmonths.index(season)
+    res_season = allmonths[sstart + len(season):sstart + 12]
+    seasons = [season, res_season]
+
     if not cube.coords('clim_season'):
-        iris.coord_categorisation.add_season(cube, 'time', name='clim_season')
+        iris.coord_categorisation.add_season(cube,
+                                             'time',
+                                             name='clim_season',
+                                             seasons=seasons)
     if not cube.coords('season_year'):
         iris.coord_categorisation.add_season_year(cube,
                                                   'time',
-                                                  name='season_year')
-    return cube.extract(iris.Constraint(clim_season=season.lower()))
+                                                  name='season_year',
+                                                  seasons=seasons)
+
+    return cube.extract(iris.Constraint(clim_season=season))
 
 
 def extract_month(cube, month):
-    """
-    Slice cube to get only the data belonging to a specific month.
+    """Slice cube to get only the data belonging to a specific month.
 
     Parameters
     ----------
@@ -152,14 +189,14 @@ def extract_month(cube, month):
     if month not in range(1, 13):
         raise ValueError('Please provide a month number between 1 and 12.')
     if not cube.coords('month_number'):
-        iris.coord_categorisation.add_month_number(cube, 'time',
+        iris.coord_categorisation.add_month_number(cube,
+                                                   'time',
                                                    name='month_number')
     return cube.extract(iris.Constraint(month_number=month))
 
 
 def get_time_weights(cube):
-    """
-    Compute the weighting of the time axis.
+    """Compute the weighting of the time axis.
 
     Parameters
     ----------
@@ -182,9 +219,52 @@ def get_time_weights(cube):
     return time_weights
 
 
-def daily_statistics(cube, operator='mean'):
+def hourly_statistics(cube, hours, operator='mean'):
+    """Compute hourly statistics.
+
+    Chunks time in x hours periods and computes statistics over them.
+
+    Parameters
+    ----------
+    cube: iris.cube.Cube
+        input cube.
+
+    hours: int
+        Number of hours per period. Must be a divisor of 24
+        (1, 2, 3, 4, 6, 8, 12)
+
+    operator: str, optional
+        Select operator to apply.
+        Available operators: 'mean', 'median', 'std_dev', 'sum', 'min', 'max'
+
+    Returns
+    -------
+    iris.cube.Cube
+        Hourly statistics cube
     """
-    Compute daily statistics.
+    if not cube.coords('hour_group'):
+        iris.coord_categorisation.add_categorised_coord(
+            cube,
+            'hour_group',
+            'time',
+            lambda coord, value: coord.units.num2date(value).hour // hours,
+            units='1')
+    if not cube.coords('day_of_year'):
+        iris.coord_categorisation.add_day_of_year(cube, 'time')
+    if not cube.coords('year'):
+        iris.coord_categorisation.add_year(cube, 'time')
+
+    operator = get_iris_analysis_operation(operator)
+    cube = cube.aggregated_by(['hour_group', 'day_of_year', 'year'], operator)
+
+    cube.remove_coord('hour_group')
+    cube.remove_coord('day_of_year')
+    cube.remove_coord('year')
+    return cube
+
+
+def daily_statistics(cube, operator='mean'):
+    """Compute daily statistics.
 
     Chunks time in daily periods and computes statistics over them;
 
@@ -217,8 +297,7 @@ def daily_statistics(cube, operator='mean'):
 
 
 def monthly_statistics(cube, operator='mean'):
-    """
-    Compute monthly statistics.
+    """Compute monthly statistics.
 
     Chunks time in monthly periods and computes statistics over them;
 
@@ -247,11 +326,12 @@ def monthly_statistics(cube, operator='mean'):
     return cube
 
 
-def seasonal_statistics(cube, operator='mean'):
-    """
-    Compute seasonal statistics.
+def seasonal_statistics(cube,
+                        operator='mean',
+                        seasons=('DJF', 'MAM', 'JJA', 'SON')):
+    """Compute seasonal statistics.
 
-    Chunks time in 3-month periods and computes statistics over them;
+    Chunks time seasons and computes statistics over them.
 
     Parameters
     ----------
@@ -263,47 +343,76 @@ def seasonal_statistics(cube, operator='mean'):
         Available operators: 'mean', 'median', 'std_dev', 'sum', 'min',
         'max', 'rms'
 
+    seasons: list or tuple of str, optional
+        Seasons to build. Available: ('DJF', 'MAM', 'JJA', SON') (default)
+        and all sequentially correct combinations holding every month
+        of a year: e.g. ('JJAS','ONDJFMAM'), or less in case of prior season
+        extraction.
+
     Returns
     -------
     iris.cube.Cube
         Seasonal statistic cube
     """
+    seasons = tuple([sea.upper() for sea in seasons])
+
+    if any([len(sea) < 2 for sea in seasons]):
+        raise ValueError(
+            f"Minimum of 2 month is required per Seasons: {seasons}.")
+
     if not cube.coords('clim_season'):
-        iris.coord_categorisation.add_season(cube, 'time', name='clim_season')
+        iris.coord_categorisation.add_season(cube,
+                                             'time',
+                                             name='clim_season',
+                                             seasons=seasons)
+    else:
+        old_seasons = list(set(cube.coord('clim_season').points))
+        if not all([osea in seasons for osea in old_seasons]):
+            raise ValueError(
+                f"Seasons {seasons} do not match prior season extraction "
+                f"{old_seasons}.")
+
     if not cube.coords('season_year'):
         iris.coord_categorisation.add_season_year(cube,
                                                   'time',
-                                                  name='season_year')
+                                                  name='season_year',
+                                                  seasons=seasons)
 
     operator = get_iris_analysis_operation(operator)
 
     cube = cube.aggregated_by(['clim_season', 'season_year'], operator)
 
     # CMOR Units are days so we are safe to operate on days
-    # Ranging on [90, 92] days makes this calendar-independent
-    def spans_three_months(time):
-        """
-        Check for three months.
+    # Ranging on [29, 31] days makes this calendar-independent
+    # the only season this could not work is 'F' but this raises an
+    # ValueError
+    def spans_full_season(cube):
+        """Check for all month present in the season.
 
         Parameters
         ----------
-        time: iris.DimCoord
-            cube time coordinate
+        cube: iris.cube.Cube
+            input cube.
 
         Returns
         -------
         bool
-            truth statement if time bounds are 90+2 days.
+            truth statement if time bounds are within (month*29, month*31)
         """
-        return 90 <= (time.bound[1] - time.bound[0]).days <= 92
+        time = cube.coord('time')
+        num_days = [(tt.bounds[0, 1] - tt.bounds[0, 0]) for tt in time]
 
-    three_months_bound = iris.Constraint(time=spans_three_months)
-    return cube.extract(three_months_bound)
+        seasons = cube.coord('clim_season').points
+        tar_days = [(len(sea) * 29, len(sea) * 31) for sea in seasons]
+
+        return [dt[0] <= dn <= dt[1] for dn, dt in zip(num_days, tar_days)]
+
+    full_seasons = spans_full_season(cube)
+    return cube[full_seasons]
 
 
 def annual_statistics(cube, operator='mean'):
-    """
-    Compute annual statistics.
+    """Compute annual statistics.
 
     Note that this function does not weight the annual mean if
     uneven time periods are present. Ie, all data inside the year
@@ -335,8 +444,7 @@ def annual_statistics(cube, operator='mean'):
 
 
 def decadal_statistics(cube, operator='mean'):
-    """
-    Compute decadal statistics.
+    """Compute decadal statistics.
 
     Note that this function does not weight the decadal mean if
     uneven time periods are present. Ie, all data inside the decade
@@ -375,9 +483,11 @@ def decadal_statistics(cube, operator='mean'):
     return cube.aggregated_by('decade', operator)
 
 
-def climate_statistics(cube, operator='mean', period='full'):
-    """
-    Compute climate statistics with the specified granularity.
+def climate_statistics(cube,
+                       operator='mean',
+                       period='full',
+                       seasons=('DJF', 'MAM', 'JJA', 'SON')):
+    """Compute climate statistics with the specified granularity.
 
     Computes statistics for the whole dataset. It is possible to get them for
     the full period or with the data grouped by day, month or season
@@ -397,6 +507,9 @@ def climate_statistics(cube, operator='mean', period='full'):
         Available periods: 'full', 'season', 'seasonal', 'monthly', 'month',
         'mon', 'daily', 'day'
 
+    seasons: list or tuple of str, optional
+        Seasons to use if needed. Defaults to ('DJF', 'MAM', 'JJA', 'SON')
+
     Returns
     -------
     iris.cube.Cube
@@ -410,8 +523,7 @@ def climate_statistics(cube, operator='mean', period='full'):
             time_weights = get_time_weights(cube)
             if time_weights.min() == time_weights.max():
                 # No weighting needed.
-                cube = cube.collapsed('time',
-                                      operator_method)
+                cube = cube.collapsed('time', operator_method)
             else:
                 cube = cube.collapsed('time',
                                       operator_method,
@@ -420,22 +532,25 @@ def climate_statistics(cube, operator='mean', period='full'):
             cube = cube.collapsed('time', operator_method)
         return cube
 
-    clim_coord = _get_period_coord(cube, period)
+    clim_coord = _get_period_coord(cube, period, seasons)
     operator = get_iris_analysis_operation(operator)
     clim_cube = cube.aggregated_by(clim_coord, operator)
     clim_cube.remove_coord('time')
     if clim_cube.coord(clim_coord.name()).is_monotonic():
         iris.util.promote_aux_coord_to_dim_coord(clim_cube, clim_coord.name())
     else:
-        clim_cube = iris.cube.CubeList(
-            clim_cube.slices_over(clim_coord.name())).merge_cube()
+        clim_cube = iris.cube.CubeList(clim_cube.slices_over(
+            clim_coord.name())).merge_cube()
     cube.remove_coord(clim_coord)
     return clim_cube
 
 
-def anomalies(cube, period, reference=None, standardize=False):
-    """
-    Compute anomalies using a mean with the specified granularity.
+def anomalies(cube,
+              period,
+              reference=None,
+              standardize=False,
+              seasons=('DJF', 'MAM', 'JJA', 'SON')):
+    """Compute anomalies using a mean with the specified granularity.
 
     Computes anomalies based on daily, monthly, seasonal or yearly means for
     the full available period
@@ -458,6 +573,8 @@ def anomalies(cube, period, reference=None, standardize=False):
     standardize: bool, optional
         If True standardized anomalies are calculated
 
+    seasons: list or tuple of str, optional
+        Seasons to use if needed. Defaults to ('DJF', 'MAM', 'JJA', 'SON')
 
     Returns
     -------
@@ -468,19 +585,23 @@ def anomalies(cube, period, reference=None, standardize=False):
         reference_cube = cube
     else:
         reference_cube = extract_time(cube, **reference)
-    reference = climate_statistics(reference_cube, period=period)
+    reference = climate_statistics(reference_cube,
+                                   period=period,
+                                   seasons=seasons)
     if period in ['full']:
         metadata = copy.deepcopy(cube.metadata)
         cube = cube - reference
         cube.metadata = metadata
         if standardize:
-            cube_stddev = climate_statistics(
-                cube, operator='std_dev', period=period)
+            cube_stddev = climate_statistics(cube,
+                                             operator='std_dev',
+                                             period=period,
+                                             seasons=seasons)
             cube = cube / cube_stddev
             cube.units = '1'
         return cube
 
-    cube = _compute_anomalies(cube, reference, period)
+    cube = _compute_anomalies(cube, reference, period, seasons)
 
     # standardize the results if requested
     if standardize:
@@ -493,17 +614,16 @@ def anomalies(cube, period, reference=None, standardize=False):
             raise ValueError(
                 "Cannot safely apply preprocessor to this dataset, "
                 "since the full time period of this dataset is not "
-                f"a multiple of the period '{period}'"
-            )
+                f"a multiple of the period '{period}'")
         cube.data = cube.core_data() / da.concatenate(
             [cube_stddev.core_data() for _ in range(int(reps))], axis=tdim)
         cube.units = '1'
     return cube
 
 
-def _compute_anomalies(cube, reference, period):
-    cube_coord = _get_period_coord(cube, period)
-    ref_coord = _get_period_coord(reference, period)
+def _compute_anomalies(cube, reference, period, seasons):
+    cube_coord = _get_period_coord(cube, period, seasons)
+    ref_coord = _get_period_coord(reference, period, seasons)
 
     data = cube.core_data()
     cube_time = cube.coord('time')
@@ -523,7 +643,7 @@ def _compute_anomalies(cube, reference, period):
     return cube
 
 
-def _get_period_coord(cube, period):
+def _get_period_coord(cube, period, seasons):
     """Get periods."""
     if period in ['daily', 'day']:
         if not cube.coords('day_of_year'):
@@ -535,14 +655,15 @@ def _get_period_coord(cube, period):
         return cube.coord('month_number')
     if period in ['seasonal', 'season']:
         if not cube.coords('season_number'):
-            iris.coord_categorisation.add_season_number(cube, 'time')
+            iris.coord_categorisation.add_season_number(cube,
+                                                        'time',
+                                                        seasons=seasons)
         return cube.coord('season_number')
     raise ValueError(f"Period '{period}' not supported")
 
 
 def regrid_time(cube, frequency):
-    """
-    Align time axis for cubes so they can be subtracted.
+    """Align time axis for cubes so they can be subtracted.
 
     Operations on time units, time points and auxiliary
     coordinates so that any cube from cubes can be subtracted from any
@@ -566,9 +687,7 @@ def regrid_time(cube, frequency):
     # standardize time points
     time_c = [cell.point for cell in cube.coord('time').cells()]
     if frequency == 'yr':
-        time_cells = [
-            datetime.datetime(t.year, 7, 1, 0, 0, 0) for t in time_c
-        ]
+        time_cells = [datetime.datetime(t.year, 7, 1, 0, 0, 0) for t in time_c]
     elif frequency == 'mon':
         time_cells = [
             datetime.datetime(t.year, t.month, 15, 0, 0, 0) for t in time_c
@@ -594,8 +713,8 @@ def regrid_time(cube, frequency):
         ]
 
     cube.coord('time').points = [
-        cube.coord('time').units.date2num(cl)
-        for cl in time_cells]
+        cube.coord('time').units.date2num(cl) for cl in time_cells
+    ]
 
     # uniformize bounds
     cube.coord('time').bounds = None
@@ -619,12 +738,10 @@ def regrid_time(cube, frequency):
 
 
 def low_pass_weights(window, cutoff):
-    """
-    Calculate weights for a low pass Lanczos filter.
+    """Calculate weights for a low pass Lanczos filter.
 
     Method borrowed from `iris example
-    <https://scitools.org.uk/iris/docs/latest/examples/General/
-    SOI_filtering.html?highlight=running%20mean>`_
+    <https://scitools-iris.readthedocs.io/en/latest/generated/gallery/general/plot_SOI_filtering.html?highlight=running%20mean>`_
 
     Parameters
     ----------
@@ -652,22 +769,21 @@ def low_pass_weights(window, cutoff):
     return weights[1:-1]
 
 
-def timeseries_filter(cube, window, span,
-                      filter_type='lowpass', filter_stats='sum'):
-    """
-    Apply a timeseries filter.
+def timeseries_filter(cube,
+                      window,
+                      span,
+                      filter_type='lowpass',
+                      filter_stats='sum'):
+    """Apply a timeseries filter.
 
     Method borrowed from `iris example
-    <https://scitools.org.uk/iris/docs/latest/examples/General/
-    SOI_filtering.html?highlight=running%20mean>`_
+    <https://scitools-iris.readthedocs.io/en/latest/generated/gallery/general/plot_SOI_filtering.html?highlight=running%20mean>`_
 
     Apply each filter using the rolling_window method used with the weights
     keyword argument. A weighted sum is required because the magnitude of
     the weights are just as important as their relative sizes.
 
-    See also the `iris rolling window
-    <https://scitools.org.uk/iris/docs/v2.0/iris/iris/
-    cube.html#iris.cube.Cube.rolling_window>`_
+    See also the iris rolling window :obj:`iris.cube.Cube.rolling_window`.
 
     Parameters
     ----------
@@ -707,13 +823,14 @@ def timeseries_filter(cube, window, span,
 
     # Construct weights depending on frequency
     # TODO implement more filters!
-    supported_filters = ['lowpass', ]
+    supported_filters = [
+        'lowpass',
+    ]
     if filter_type in supported_filters:
         if filter_type == 'lowpass':
             wgts = low_pass_weights(window, 1. / span)
     else:
-        raise NotImplementedError(
-            "Filter type {} not implemented, \
+        raise NotImplementedError("Filter type {} not implemented, \
             please choose one of {}".format(filter_type,
                                             ", ".join(supported_filters)))
 
@@ -725,3 +842,107 @@ def timeseries_filter(cube, window, span,
                                weights=wgts)
 
     return cube
+
+
+def resample_hours(cube, interval, offset=0):
+    """Convert x-hourly data to y-hourly by eliminating extra timesteps.
+
+    Convert x-hourly data to y-hourly (y > x) by eliminating the extra
+    timesteps. This is intended to be used only with instantaneous values.
+
+    For example:
+
+    - resample_hours(cube, interval=6): Six-hourly intervals at 0:00, 6:00,
+      12:00, 18:00.
+
+    - resample_hours(cube, interval=6, offset=3): Six-hourly intervals at
+      3:00, 9:00, 15:00, 21:00.
+
+    - resample_hours(cube, interval=12, offset=6): Twelve-hourly intervals
+      at 6:00, 18:00.
+
+    Parameters
+    ----------
+    cube: iris.cube.Cube
+        Input cube.
+    interval: int
+        The period (hours) of the desired data.
+    offset: int, optional
+        The firs hour (hours) of the desired data.
+
+    Returns
+    -------
+    iris.cube.Cube
+        Cube with the new frequency.
+
+    Raises
+    ------
+    ValueError:
+        The specified frequency is not a divisor of 24.
+    """
+    allowed_intervals = (1, 2, 3, 4, 6, 12)
+    if interval not in allowed_intervals:
+        raise ValueError(
+            f'The number of hours must be one of {allowed_intervals}')
+    if offset >= interval:
+        raise ValueError(f'The offset ({offset}) must be lower than '
+                         f'the interval ({interval})')
+    time = cube.coord('time')
+    cube_period = time.cell(1).point - time.cell(0).point
+    if cube_period.total_seconds() / 3600 >= interval:
+        raise ValueError(f"Data period ({cube_period}) should be lower than "
+                         f"the interval ({interval})")
+    hours = range(0 + offset, 24, interval)
+    select_hours = iris.Constraint(time=lambda cell: cell.point.hour in hours)
+    return cube.extract(select_hours)
+
+
+def resample_time(cube, month=None, day=None, hour=None):
+    """Change frequency of data by resampling it.
+
+    Converts data from one frequency to another by extracting the timesteps
+    that match the provided month, day and/or hour. This is meant to be used
+    with instantaneous values when computing statistics is not desired.
+
+    For example:
+
+    - resample_time(cube, hour=6): Daily values taken at 6:00.
+
+    - resample_time(cube, day=15, hour=6): Monthly values taken at 15th
+      6:00.
+
+    - resample_time(cube, month=6): Yearly values, taking in June
+
+    - resample_time(cube, month=6, day=1): Yearly values, taking 1st June
+
+    The condition must yield only one value per interval: the last two samples
+    above will produce yearly data, but the first one is meant to be used to
+    sample from monthly output and the second one will work better with daily.
+
+    Parameters
+    ----------
+    cube: iris.cube.Cube
+        Input cube.
+    month: int, optional
+        Month to extract
+    day: int, optional
+        Day to extract
+    hour: int, optional
+        Hour to extract
+
+    Returns
+    -------
+    iris.cube.Cube
+        Cube with the new frequency.
+    """
+    def compare(cell):
+        date = cell.point
+        if month is not None and month != date.month:
+            return False
+        if day is not None and day != date.day:
+            return False
+        if hour is not None and hour != date.hour:
+            return False
+        return True
+
+    return cube.extract(iris.Constraint(time=compare))
