@@ -4,24 +4,28 @@ import numpy as np
 import pytest
 from cf_units import Unit
 
-from esmvalcore.cmor._fixes.shared import (_get_altitude_to_pressure_func,
-                                           add_aux_coords_from_cubes,
-                                           add_plev_from_altitude,
-                                           add_scalar_depth_coord,
-                                           add_scalar_height_coord,
-                                           add_scalar_typeland_coord,
-                                           add_scalar_typesea_coord,
-                                           add_sigma_factory,
-                                           altitude_to_pressure,
-                                           cube_to_aux_coord, fix_bounds,
-                                           get_bounds_cube, round_coordinates)
+from esmvalcore.cmor._fixes.shared import (
+    add_altitude_from_plev,
+    add_aux_coords_from_cubes,
+    add_plev_from_altitude,
+    add_scalar_depth_coord,
+    add_scalar_height_coord,
+    add_scalar_typeland_coord,
+    add_scalar_typesea_coord,
+    add_sigma_factory,
+    cube_to_aux_coord,
+    fix_bounds,
+    get_altitude_to_pressure_func,
+    get_bounds_cube,
+    get_pressure_to_altitude_func,
+    round_coordinates,
+)
 from esmvalcore.iris_helpers import var_name_constraint
 
 
-@pytest.mark.parametrize('func', [altitude_to_pressure,
-                                  _get_altitude_to_pressure_func()])
-def test_altitude_to_pressure_func(func):
+def test_altitude_to_pressure_func():
     """Test altitude to pressure function."""
+    func = get_altitude_to_pressure_func()
     assert callable(func)
     np.testing.assert_allclose(func(-6000.0), 196968.01058487315)
     np.testing.assert_allclose(func(-5000.0), 177687.0)
@@ -33,6 +37,20 @@ def test_altitude_to_pressure_func(func):
                                [101325.0, 100129.0])
 
 
+def test_pressure_to_altitude_func():
+    """Test pressure to altitude function."""
+    func = get_pressure_to_altitude_func()
+    assert callable(func)
+    np.testing.assert_allclose(func(200000.0), -6166.332306480035)
+    np.testing.assert_allclose(func(177687.0), -5000.0)
+    np.testing.assert_allclose(func(101325.0), 0.0, atol=1.0e-7)
+    np.testing.assert_allclose(func(1000.0), 31054.63120206961)
+    np.testing.assert_allclose(func(75.9448), 50000)
+    np.testing.assert_allclose(func(0.1), 91607.36011892557)
+    np.testing.assert_allclose(func(np.array([101325.0, 177687.0])),
+                               [0.0, -5000.0], atol=1.0e-7)
+
+
 TEST_ADD_AUX_COORDS_FROM_CUBES = [
     ({}, 1),
     ({'x': ()}, 0),
@@ -82,20 +100,31 @@ def test_add_aux_coords_from_cubes(coord_dict, output):
 
 
 ALT_COORD = iris.coords.AuxCoord([0.0], bounds=[[-100.0, 500.0]],
+                                 var_name='alt', long_name='altitude',
                                  standard_name='altitude', units='m')
+ALT_COORD_NB = iris.coords.AuxCoord([0.0], var_name='alt',
+                                    long_name='altitude',
+                                    standard_name='altitude', units='m')
 ALT_COORD_KM = iris.coords.AuxCoord([0.0], bounds=[[-0.1, 0.5]],
+                                    var_name='alt', long_name='altitude',
                                     standard_name='altitude', units='km')
 P_COORD = iris.coords.AuxCoord([101325.0], bounds=[[102532.0, 95460.8]],
                                var_name='plev', standard_name='air_pressure',
                                long_name='pressure', units='Pa')
+P_COORD_NB = iris.coords.AuxCoord([101325.0], var_name='plev',
+                                  standard_name='air_pressure',
+                                  long_name='pressure', units='Pa')
 CUBE_ALT = iris.cube.Cube([1.0], var_name='x',
                           aux_coords_and_dims=[(ALT_COORD, 0)])
+CUBE_ALT_NB = iris.cube.Cube([1.0], var_name='x',
+                             aux_coords_and_dims=[(ALT_COORD_NB, 0)])
 CUBE_ALT_KM = iris.cube.Cube([1.0], var_name='x',
                              aux_coords_and_dims=[(ALT_COORD_KM, 0)])
 
 
 TEST_ADD_PLEV_FROM_ALTITUDE = [
     (CUBE_ALT.copy(), P_COORD.copy()),
+    (CUBE_ALT_NB.copy(), P_COORD_NB.copy()),
     (CUBE_ALT_KM.copy(), P_COORD.copy()),
     (iris.cube.Cube(0.0), None),
 ]
@@ -115,6 +144,58 @@ def test_add_plev_from_altitude(cube, output):
     add_plev_from_altitude(cube)
     air_pressure_coord = cube.coord('air_pressure')
     assert air_pressure_coord == output
+    assert cube.coords('altitude')
+
+
+P_COORD_HPA = iris.coords.AuxCoord([1013.25], bounds=[[1025.32, 954.60]],
+                                   var_name='plev',
+                                   standard_name='air_pressure',
+                                   long_name='pressure', units='hPa')
+CUBE_PLEV = iris.cube.Cube([1.0], var_name='x',
+                           aux_coords_and_dims=[(P_COORD, 0)])
+CUBE_PLEV_NB = iris.cube.Cube([1.0], var_name='x',
+                              aux_coords_and_dims=[(P_COORD_NB, 0)])
+CUBE_PLEV_HPA = iris.cube.Cube([1.0], var_name='x',
+                               aux_coords_and_dims=[(P_COORD_HPA, 0)])
+
+
+TEST_ADD_ALTITUDE_FROM_PLEV = [
+    (CUBE_PLEV.copy(), ALT_COORD.copy()),
+    (CUBE_PLEV_NB.copy(), ALT_COORD_NB.copy()),
+    (CUBE_PLEV_HPA.copy(), ALT_COORD.copy()),
+    (iris.cube.Cube(0.0), None),
+]
+
+
+@pytest.mark.parametrize('cube,output', TEST_ADD_ALTITUDE_FROM_PLEV)
+def test_add_altitude_from_plev(cube, output):
+    """Test adding of altitude coordinate."""
+    if output is None:
+        with pytest.raises(ValueError) as err:
+            add_altitude_from_plev(cube)
+        msg = ("Cannot add 'altitude' coordinate, 'air_pressure' coordinate "
+               "not available")
+        assert str(err.value) == msg
+        return
+    assert not cube.coords('altitude')
+    add_altitude_from_plev(cube)
+    altitude_coord = cube.coord('altitude')
+    metadata_list = [
+        'var_name',
+        'standard_name',
+        'long_name',
+        'units',
+        'attributes',
+    ]
+    for attr in metadata_list:
+        assert getattr(altitude_coord, attr) == getattr(output, attr)
+    np.testing.assert_allclose(altitude_coord.points, output.points, atol=1e-7)
+    if output.bounds is None:
+        assert altitude_coord.bounds is None
+    else:
+        np.testing.assert_allclose(altitude_coord.bounds, output.bounds,
+                                   rtol=1e-3)
+    assert cube.coords('air_pressure')
 
 
 DIM_COORD = iris.coords.DimCoord([3.141592],
