@@ -73,6 +73,8 @@ class CoordinateInfoMock:
         self.stored_direction = "increasing"
         self.must_have_bounds = "yes"
         self.requested = []
+        self.generic_lev_coords = {}
+        self.generic_lev_name = ""
 
         valid_limits = {'lat': ('-90', '90'), 'lon': ('0', '360')}
         if name in valid_limits:
@@ -281,9 +283,23 @@ class TestCMORCheck(unittest.TestCase):
         lat_points = lat_points / 3.0 - 50.
         self.cube.remove_coord('latitude')
         iris.util.demote_dim_coord_to_aux_coord(self.cube, 'longitude')
+        lat_points = np.concatenate(
+            (
+                self.cube.coord('longitude').points[0:10] / 4,
+                self.cube.coord('longitude').points[0:10] / 4
+            ),
+            axis=0
+        )
+        lat_bounds = np.concatenate(
+            (
+                self.cube.coord('longitude').bounds[0:10] / 4,
+                self.cube.coord('longitude').bounds[0:10] / 4
+            ),
+            axis=0
+        )
         new_lat = iris.coords.AuxCoord(
-            points=self.cube.coord('longitude').points / 4,
-            bounds=self.cube.coord('longitude').bounds / 4,
+            points=lat_points,
+            bounds=lat_bounds,
             var_name='lat',
             standard_name='latitude',
             long_name='Latitude',
@@ -292,6 +308,20 @@ class TestCMORCheck(unittest.TestCase):
         self.cube.add_aux_coord(new_lat, 1)
         self._check_cube()
 
+    def test_bad_generic_level(self):
+        """Test check fails in metadata if generic level coord
+        has wrong var_name."""
+        depth_coord = CoordinateInfoMock('depth')
+        depth_coord.axis = 'Z'
+        depth_coord.generic_lev_name = 'olevel'
+        depth_coord.out_name = 'lev'
+        depth_coord.name = 'depth_coord'
+        depth_coord.long_name = 'ocean depth coordinate'
+        self.var_info.coordinates['depth'].generic_lev_coords = {
+            'depth_coord': depth_coord}
+        self.var_info.coordinates['depth'].out_name = ""
+        self._check_fails_in_metadata()
+
     def test_check_bad_var_standard_name_strict_flag(self):
         """Test check fails for a bad variable standard_name with
         --cmor-check strict."""
