@@ -1,21 +1,26 @@
 """Unit tests for the esmvalcore.preprocessor._regrid_esmpy module."""
+import sys
 from unittest import mock
 
 import cf_units
 import iris
 import numpy as np
+import pytest
 from iris.exceptions import CoordinateNotFoundError
 
 import tests
-from esmvalcore.preprocessor._regrid_esmpy import (build_regridder,
-                                                   build_regridder_2d,
-                                                   coords_iris_to_esmpy,
-                                                   cube_to_empty_field,
-                                                   get_grid,
-                                                   get_grid_representant,
-                                                   get_grid_representants,
-                                                   get_representant,
-                                                   is_lon_circular, regrid)
+from esmvalcore.preprocessor._regrid_esmpy import (
+    build_regridder,
+    build_regridder_2d,
+    coords_iris_to_esmpy,
+    cube_to_empty_field,
+    get_grid,
+    get_grid_representant,
+    get_grid_representants,
+    get_representant,
+    is_lon_circular,
+    regrid,
+)
 
 
 def identity(*args, **kwargs):
@@ -355,6 +360,9 @@ class TestHelpers(tests.Test):
         self.assert_array_equal(esmpy_lon_corners,
                                 self.expected_esmpy_lon_corners)
 
+    @pytest.mark.skipif(sys.version_info.major == 3
+                        and sys.version_info.minor == 9,
+                        reason="bug in mock.py for Python 3.9.0 and 3.9.1")
     def test_get_grid_circular(self):
         """Test building of ESMF grid from iris cube circular longitude."""
         expected_get_coords_calls = [
@@ -375,6 +383,9 @@ class TestHelpers(tests.Test):
             mg.add_item.assert_called_once_with(mock.sentinel.gi_mask,
                                                 mock.sentinel.sl_center)
 
+    @pytest.mark.skipif(sys.version_info.major == 3
+                        and sys.version_info.minor == 9,
+                        reason="bug in mock.py for Python 3.9.0 and 3.9.1")
     def test_get_grid_non_circular(self):
         """Test building of ESMF grid from iris cube non circular longitude."""
         expected_get_coords_calls = [
@@ -431,6 +442,9 @@ class TestHelpers(tests.Test):
 
     @mock.patch('ESMF.Grid', MockGrid)
     @mock.patch('ESMF.Field')
+    @pytest.mark.skipif(sys.version_info.major == 3
+                        and sys.version_info.minor == 9,
+                        reason="bug in mock.py for Python 3.9.0 and 3.9.1")
     def test_cube_to_empty_field(self, mock_field):
         """Test building of empty field from iris cube."""
         field = cube_to_empty_field(self.cube)
